@@ -1,4 +1,3 @@
-use sozu_command::buffer::fixed::Buffer;
 use pool_crate::Reset;
 use std::io::{self,Write};
 use std::cmp::{min,max};
@@ -319,7 +318,7 @@ impl BufferQueue {
     }
   }
 
-  pub fn as_iovec(&self) -> Vec<&iovec::IoVec> {
+  pub fn as_ioslice(&self) -> Vec<std::io::IoSlice> {
     let mut res = Vec::new();
 
     let it = self.output_queue.iter();
@@ -336,14 +335,11 @@ impl BufferQueue {
             continue
           }
           let end = min(start+sz, length);
-          if let Some(i) = iovec::IoVec::from_bytes(&self.buffer.data()[start..end]) {
-            //println!("iovec size: {}", i.len());
-            res.push(i);
-            start = end;
-            if end == length {
-              break;
-            }
-          } else {
+          let i = std::io::IoSlice::new(&self.buffer.data()[start..end]);
+          //println!("iovec size: {}", i.len());
+          res.push(i);
+          start = end;
+          if end == length {
             break;
           }
         }
@@ -351,14 +347,11 @@ impl BufferQueue {
           if v.is_empty() {
             continue
           }
-          if let Some(i) = iovec::IoVec::from_bytes(&v[..]) {
-            //println!("got Insert with {} bytes", v.len());
-            res.push(i);
-          } else {
-            break;
-          }
+          let i = std::io::IoSlice::new(&v[..]);
+          //println!("got Insert with {} bytes", v.len());
+          res.push(i);
         },
-        &OutputElement::Splice(_sz)  => { unimplemented!("splice not used in iovec") },
+        &OutputElement::Splice(_sz)  => { unimplemented!("splice not used in ioslice") },
       }
     }
 
@@ -480,7 +473,7 @@ impl fmt::Debug for BufferQueue {
 
 
 pub fn buf_with_capacity(capacity: usize) -> (Pool, BufferQueue) {
-  let mut pool = Pool::with_capacity(1, capacity);
+  let mut pool = Pool::with_capacity(1, capacity, 16384);
   let b = BufferQueue::with_buffer(pool.checkout().unwrap());
   (pool, b)
 }
@@ -515,13 +508,13 @@ mod tests {
     };*/
 
     // the pool will align the buffer to 16 bytes so there are trailing zeroes
-    assert_eq!(b.unparsed_data(), &b"ABCDEFGHIJ\0\0\0\0\0\0"[..]);
+    assert_eq!(b.unparsed_data(), &b"ABCDEFGHIJ\0\0\0\0\0\0\0\0\0\0"[..]);
     b.consume_parsed_data(4);
     assert_eq!(b.parsed_position, 4);
     assert_eq!(b.start_parsing_position, 4);
     assert_eq!(b.input_queue, vec!(InputElement::Slice(6)));
     println!("TEST[{}]", line!());
-    assert_eq!(b.unparsed_data(), &b"EFGHIJ\0\0\0\0\0\0"[..]);
+    assert_eq!(b.unparsed_data(), &b"EFGHIJ\0\0\0\0\0\0\0\0\0\0"[..]);
     println!("TEST[{}]", line!());
 
     b.slice_output(4);
